# Cushion.js

This is a data model abstraction layer built for Node.js app. It is very heavily informed by the [resourceful](http://github.com/flatiron/resourceful) from Nodejitsu and the external API is almost identical. Cushion.js started out as a project built for use with Riak, then changed to CouchBase. The important thing is that the data access layer has been abstracted in such a way now, that probably almost any NoSQL data store could be used with the addition of a simple adapter.

## Installation

This module has not yet been publish to npm. Installation will need to be made from this repository.

```bash
$ npm install git://github.com/brianedgerton/cushion.js.git
```

## API

When resources are defined, there are two types of related objects that are created. The __factory__ object contains methods which return __instances__ of the resource. We will look at the API's of both types of objects.

### Factory Methods

`Resource.all( options, callback )` : Fetches a listing of resources.

`Resource.get( id, options, callback )` : Fetches a single resource by `id`

`Resource.update( id, data, options, callback )` : Updates the resource with the specified `id` using the `data` object

`Resource.destroy( id, options, callback )` : Destroys a resource

`Resource.create( data, options, callback )`: Creates a new resource with the specified `data`

`Resource.new( data )` : Returns an instance of the resource with the `data` object as the properties

## Instance Methods

`instance.save( callback )` : Persists the instance's properties to the database

`instance.destroy( callback )` : Removes the instance from the database

## Validation

Cushion.js uses [revalidator](http://github.com/flatiron/revalidator) to handle validation. The the schema of the resource is declared as a standard revalidator schema object.

```javascript
Resource.fields({
    name: {
        type: 'string',
        required: true
    },
    slug: {
        type: 'string',
        required: true
    },
    description: {
        type: 'string'
    }
});
```

__Please see the revalidator documentation for more information on how to declare the schema.__

## Hooks

Cushion.js provides several hooks in the resource saving and deleting workflows. Here is an example:

```javascript
Resource.before( 'save', function( done ) {
    // Access resource instance properties via 'this'
    this.slug = slugify( this.name );

    // If an error is passed to the done callback, saving will halt and an error will be passed back
    done();
});
```

`Resource.before( 'save', callback )` : Adds callback to queue that is run before validation/saving. Returning an error will stop saving.

`Resource.after( 'save', callback )` : Adds callback to queue that is run after saving.

`Resource.before( 'destroy', callback )` : Adds callback to queue that is run before deletion. Returning an error will stop deletion.

`Resource.after( 'destroy', callback )` : Adds callback to queue that is run after deletion.
