var _ = require( "lodash" ),
    couchbase = require( "couchbase" ),
    machina = require( "machina" )( _ ),
    createFsm = function( options ) {

        var magicMethods = [
            "touch",
            "get",
            "set",
            "remove",
            "replace",
            "add",
            "append",
            "prepend",
            "view",
            "incr",
            "decr",
            "observe",
            "endure",
            "setDesignDoc",
            "getDesignDoc",
            "deleteDesignDoc",
            "getVersion",
            "strError"
        ],
            fsm = {
                client: null,

                connect: function() {
                    var self = this;
                    couchbase.connect( options, function( err, client ) {
                        if ( err ) {
                            self.emit( 'fail' );
                            return;
                        }
                        self.client = client;
                        self.transition( 'connected' );
                    });
                },

                initialState: 'disconnected',

                states: {
                    disconnected: {

                        _onEnter: function() {
                            this.connect();
                        },

                        "*": function() {
                            this.deferUntilTransition( 'connected' );
                        }

                    },

                    connected: {
                        "*": function( method, args ) {
                            this.client[ method ].apply( this.client, args );
                        }
                    }
                }

            };

        _.each( magicMethods, function( method ) {
            fsm[ method ] = function() {
                var args = Array.prototype.slice.call( arguments );
                this.handle.call( this, method, args );
            };
        });

        var ClientFsm = new machina.Fsm( fsm );

        return ClientFsm;
    };


module.exports = function( options ) {
    return createFsm( options );
};