var _ = require( "lodash" ),
    revalidator = require( "revalidator" ),
    Monologue = require( "monologue.js" )( _ ),
    relationships = require( "./relationships.js" ),
    mixins = require( "./mixins.js" ),
    noop = function(){};

module.exports = function( config ) {

    var defaults = {
        dataRequestChannel: 'resource.data.request',
        dataResponseChannel: 'resource.data.response',
        dataListenChannel: 'resource.data.request'
    };

    config = config ? _.defaults( config, defaults ) : _.clone( defaults );

    var resource = {
        config: config,
        registry: {},
        relationships: relationships,
        mixins: mixins,
        bus: config.bus || require( "postal" )( _ ),
        addRelationship: relationships.addRelationship,
        define: function( name, init ) {
            var factory = resource.registry[ name ] = new resource.factory( name );
            init.call( factory );

            resource.emit( 'resource.define', { name: name, factory: factory } );
            return factory;
        },
        get: function( name ) {
            if ( resource.registry[ name ] ) {
                return resource.registry[ name ];
            } else {
                throw new Error( 'Resource not found: ' + name );
            }
        },
        relate: function( msg, env ) {
            var name = msg.name,
                reg = resource.registry,
                rels = _.filter( relationships.registry, function( rel ) {
                    if ( rel.parent === name && reg[ rel.child ] ) {
                        return true;
                    }

                    if ( rel.child === name && reg[ rel.parent ] ) {
                        return true;
                    }

                    return false;
                });

            if ( rels.length ) {
                for ( var i = 0, j = rels.length; i < j; i++ ) {
                    relationships.createRelationship( rels[i], resource );
                }
            }
        },
        setup: function() {
            resource.on( 'resource.define', resource.relate );
        },
        reset: function() {
            resource.registry = [];
            resource.relationships.registry = [];
        }

    };

    _.extend( resource, Monologue.prototype );

    resource.setup();

    var engine = config.engine || 'couchbase';
    resource.Client = require( "./engines/" + engine + ".js" ),
    resource.Client.listen( { bus: resource.bus, config: resource.config } );
    resource.instance = require( "./instance" )( resource ),
    resource.factory = require( "./factory.js" )( resource );

    return resource;
};