var fs = require( "fs" );

module.exports = function( _, resource, fixturePath ) {

    var resourceDataChannel = 'resource.data.request';

    var DbMock = function() {
        var bus = resource.bus,
            subs = [],
            buckets = [],
            getBucket = function( name ) {
                if ( ! buckets[ name ] ) {
                    var contents = JSON.parse( fs.readFileSync( fixturePath + "/" + name + ".json" ) );
                    buckets[ name ] = contents.data;
                }

                return buckets[ name ];
            };

        this.clear = function() {
            _.each( subs, function( sub ) {
                if( sub.unsubscribe ) {
                    sub.unsubscribe();
                }
            });

            subs = [];
        };

        var sub1 = bus.subscribe({
            channel: resourceDataChannel,
            topic: "init",
            callback: function( data, envelope ) {
                var bucket = getBucket( data.payload.name );
            }
        });
        
        var sub2 = bus.subscribe({
            channel: resourceDataChannel,
            topic: "get",
            callback: function( data, envelope ) {

                var bucket = getBucket( data.payload.bucket );
                if ( _.isArray( data.payload.id ) ) {
                    var results = _.filter( bucket, function( r ) {
                        return _.contains( data.payload.id, r.id );
                    });

                    bus.publish({
                        channel: data.replyTo,
                        topic: 'get',
                        data: {
                            err: null,
                            results: results
                        }
                    });

                } else {

                    var result = _.find( bucket, function( r ) {
                        return r.id == data.payload.id;
                    });

                    bus.publish({
                        channel: data.replyTo,
                        topic: 'get',
                        data: {
                            err: null,
                            results: result
                        }
                    });

                }
            }
        });

        var sub3 = bus.subscribe({
            channel: resourceDataChannel,
            topic: "all",
            callback: function( data, envelope ) {
                var bucket = getBucket( data.payload.bucket );

                bus.publish({
                    channel: data.replyTo,
                    topic: 'all',
                    data: {
                        err: null,
                        results: bucket
                    }
                });

            }
        });

        var sub4 = bus.subscribe({
            channel: resourceDataChannel,
            topic: "find",
            callback: function( data, envelope ) {
                var client = Client.getClient( data.bucket ),
                    split = data.path.split( "/" );

                bus.publish({
                    channel: data.replyTo,
                    topic: 'find',
                    data: {
                        err: null,
                        results: bucket
                    }
                });

            }
        });

        var sub5 = bus.subscribe({
            channel: resourceDataChannel,
            topic: "save",
            callback: function( data, envelope ) {
                var client = Client.getClient( data.bucket );
                client.handle( 'set', [ data.id, data.data, data.callback ] );
            }
        });

        var sub6 = bus.subscribe({
            channel: resourceDataChannel,
            topic: "destroy",
            callback: function( data, envelope ) {
                var client = Client.getClient( data.bucket );
                client.handle( 'remove', [ data.id, data.callback ] );
            }
        });

        subs.push( sub1 );
        subs.push( sub2 );
        subs.push( sub3 );
        subs.push( sub4 );
        subs.push( sub5 );
        subs.push( sub6 );

    };

    return DbMock;
};