var fs = require( "fs" ),
	db = require( "couchbase" ),
	_ = require( "lodash" ),
	filters = {},
	sources = [
		"articles.json",
		"categories.json",
		"comments.json"
	],
	buckets = [
		"articles",
		"categories",
		"comments"
	];


var installer = function( config, options ) {

	options = options || {};
	options.debug = options.debug === false ? options.debug : true;

	var couch_config = config,
		db_conn = {
			hosts: couch_config.hosts,
			user: couch_config.user,
			password: couch_config.password
		},
		fixturePath = options.fixturePath || __dirname;

	var api = {
		run: function( done ) {
			if ( options.debug ) console.log( "Clearing database" );
			api.clearDB( function() {
				if ( options.debug )  console.log("Database cleared");
				api.importData( function() {
					if ( options.debug )  console.log("Data imported");
					if ( done ) {
						done();
					} else {
						process.exit();
					}
				});

			});

		},

		clearDB: function( done ) {

			if( ! buckets.length ) {
				done();
			}

			var bucket_length = buckets.length,
				bucket_counter = 0,
				bucket_done = function() {
					bucket_counter++;
					if ( bucket_counter === bucket_length ) {
						done();
					}
				};

			_.each( buckets, function( bucket ) {
				var keys = [];
				db_conn.bucket = bucket;
				db.connect( db_conn, function( err, conn ) {
					conn.view( bucket, 'all', {}, function( err, docs ) {
						var key_length = docs ? docs.length : 0,
							key_counter = 0,
							key_done = function() {
								key_counter++;
								if ( key_counter === key_length ) {
									bucket_done();
								}
							};

						if ( ! key_length ) {
							bucket_done();
						}

						_.each( docs, function( doc ) {
							if ( options.debug ) console.log("Removing ", doc.id, " from bucket ", bucket);
							conn.remove( doc.id, function( err, response ) {
								key_done();
							} );

						});
					});


				});

			});


		},

		importData: function( done ) {

			var sources_length = sources.length,
				sources_counter = 0,
				source_done = function() {
					sources_counter++;
					if ( sources_counter === sources.length ) {
						done();
					}
				};

			if ( ! sources_length ) {
				done();
			}

			_.each( sources, function( source ) {

				fs.readFile( fixturePath + "/" + source, 'utf-8', function( err, contents ) {
					contents = JSON.parse( contents );
					var bucket = contents.bucket,
						data = contents.data,
						bucket_filters = contents.filters,
						record_length = data.length,
						record_counter = 0,
						record_done = function() {
							record_counter++;
							if ( record_counter === record_length ) {
								source_done();
							}
						};

					if ( ! record_length ) {
						return source_done();
					}

					db_conn.bucket = bucket;
					db.connect( db_conn, function( err, conn ) {
						_.each( data, function( record ) {
							var id = record.id,
								data = _.omit( record, 'id' );

							if ( bucket_filters ) {
								_.each( data, function( val, key ) {
									var filter_name = bucket_filters[ key ];
									if ( filter_name && filters[ filter_name ] ) {
										data[ key ] = filters[ filter_name ]( val );
									}
								});
							}
							conn.set( id, data, record_done );
						});
					});
				});

			});

		}
	};
	return api;
};

module.exports = installer;

// If this file was called directly, bootstrap the app and run the installer
if ( require.main === module ) {
	require( __dirname + "/../" + "_setup.js" )( global, false );
		i = installer( global.couchbase_config );

	i.run();
}
