require("./_setup.js")( global );

var r = require( baseUrl + "/relationships.js" );

describe( "Relationships", function() {

    describe( "addRelationship", function() {

        it( "should add a relationship to the registry", function() {

            r.registry = [];

            r.addRelationship( 'category', 'article', {} );
            expect( r.registry ).to.eql( [ { parent: 'category', child: 'article', options: {} } ] );

            r.addRelationship( 'category', 'article', {} );
            expect( r.registry ).to.eql( [ { parent: 'category', child: 'article', options: {} } ] );

            r.addRelationship( 'article', 'comment', { cascadeDelete: false } );
            expect( r.registry ).to.eql( [
                { parent: 'category', child: 'article', options: {} },
                { parent: 'article', child: 'comment', options: { cascadeDelete: false } }
            ] );

        });

    });

    describe( "createRelationship", function() {

        it( "construct parent and child objects for adding methods", function() {

            var resource = {
                get: function( name ) {
                    return {};
                }
            };

            r.addChildMethods = sinon.stub( r, 'addChildMethods' );

            r.registry = [];

            r.createRelationship( { parent: 'Category', child: 'article', options: { cascadeDelete: false } }, resource );

            var args = r.addChildMethods.getCall( 0 ).args;

            expect( args[0] ).to.eql( {
                name: 'category',
                plural: 'categories',
                factory: {}
            } );

            expect( args[1] ).to.eql( {
                name: 'article',
                plural: 'articles',
                factory: {}
            } );

            expect( args[2] ).to.eql( {
                cascadeDelete: false
            } );

            r.addChildMethods.restore();
        });

    });


    describe( "addChildMethods", function() {

        it( "attach methods for child retrieval to the parent factory", function() {

            var resource = {
                get: function( name ) {
                    return {};
                }
            };

            var parent = {
                name: 'category',
                plural: 'categories',
                factory: {
                    _beforeDestroy: [],
                    before: function( name, cb ) {
                        parent.factory._beforeDestroy.push( cb );
                    }
                }
            };

            var child = {
                name: 'article',
                plural: 'articles',
                factory: {}
            };

            r.addChildMethods( parent, child, {} );

            expect( parent.factory.articles ).to.be.ok();
            expect( parent.factory.article ).to.be.ok();
            expect( parent.factory._beforeDestroy.length ).to.equal( 1 );

            delete parent.factory.articles;
            delete parent.factory.article;
            parent.factory._beforeDestroy = [];

            r.addChildMethods( parent, child, { cascadeDelete: false } );

            expect( parent.factory.articles ).to.be.ok();
            expect( parent.factory.article ).to.be.ok();
            expect( parent.factory._beforeDestroy.length ).to.equal( 0 );

        });

    });

});

