require("./_setup.js")( global );

describe( "Resource", function() {

    before( function( done ) {
        resource.reset();
        done();
    });

    after( function( done ) {
        resource.reset();
        done();
    });

    var Article;

    describe( "define", function() {

        it( "should emit a define event and return a ResourceFactory instance", function( done ) {

            var define_event = false;

            resource.on( 'resource.define', function( data, env ) {
                define_event = true;
                expect( data.name ).to.equal( 'article' );
            });

            Article = resource.define( 'article', function() {

                this.configure({
                    client: {
                        hosts: couchbase_config.hosts,
                        user: couchbase_config.user,
                        password: couchbase_config.password,
                        bucket: 'articles'
                    },
                    timestamps: true
                });

                this.fields({
                    name: {
                        type: 'string',
                        required: true
                    },
                    slug: {
                        type: 'string',
                        required: true
                    },
                    description: {
                        type: 'string'
                    }
                });

            });

            expect( Article.__proto__ ).to.eql( resource.factory.prototype );
            expect( resource.registry[ 'article' ] ).to.eql( Article );
            setTimeout( function() {
                expect( define_event ).to.be.ok();
                done();
            }, 100 );

        });

    });

    describe( "get", function() {

        it( "should return a resource object", function() {
            var res = resource.get( 'article' );
            expect( res ).to.eql( Article );
        });

        it( "should throw an error if resource does not exist", function() {
            try {
                var res = resource.get( "noexisty" );
                expect( false ).to.be.ok();
            } catch( e ) {
                expect( e ).to.be.ok();
            }
        });

    });

    describe( "relate", function() {
        it( "should check to see if the resource has a relationship already declared", function() {
            resource.relationships.createRelationship = sinon.stub( resource.relationships, 'createRelationship' );

            resource.reset();

            // Artificially create category resource in registry
            resource.registry[ 'category' ] = true;

            var rel1 = {
                parent: 'category',
                child: 'article',
                options: {}
            };

            resource.relationships.registry.push( rel1 );

            resource.emit( 'resource.define', { name: 'category' } );
            expect( resource.relationships.createRelationship.called ).to.not.be.ok();

            resource.emit( 'resource.define', { name: 'comment' } );
            expect( resource.relationships.createRelationship.called ).to.not.be.ok();

            resource.registry[ 'article' ] = true;
            resource.emit( 'resource.define', { name: 'category' } );

            expect( resource.relationships.createRelationship.called ).to.be.ok();
            resource.relationships.createRelationship.restore();
        });
    });

});

